using System;
using System.Collections;

namespace IFOInfo
{
	/// <summary>
	/// Summary description for IICommandLine.
	/// </summary>
	public class IICommandLine
	{
        private string m_sErrors;
        private bool m_bValid;

		public IICommandLine(string[] args)
		{
            m_sErrors = "";
            m_bValid = true;

            // Default options
            DumpChapters = true;
            UseHMS = false;
            IFOFile = "VTS_01_0.IFO";
            PGC = -1;
            FPS = 23.976;
            UseLongestPGC = false;
            ChapterMinimum = 3;
            AlwaysDisplayPGC = false;

            int i = 0;
            ArrayList normalArgs = new ArrayList();
            NextArg nextArg = NextArg.None;

            while (i < args.Length)
            {
                if (nextArg != NextArg.None)
                {
                    switch(nextArg)
                    {
                        case NextArg.ProgramChain:
                            PGC = IIGlobal.ToInt(args[i]);
                            break;
                        case NextArg.FPS:
                            FPS = IIGlobal.ToDouble(args[i]);
                            break;
                        case NextArg.ChapterMinimum:
                            ChapterMinimum = IIGlobal.ToInt(args[i]);
                            break;
                    }
                    nextArg = NextArg.None;
                }
                else switch(args[i][0])
                {
                    case '-':
                    case '/':
                        nextArg = ParseOptions(ref i, args[i].Substring(1));
                        break;
                    default:
                        normalArgs.Add(args[i]);
                        break;
                }

                i++;
            }

            if (normalArgs.Count > 0)
                IFOFile = (string) normalArgs[0];

            if (normalArgs.Count > 1)
                AddWarning("Ignoring extra command line arguments");
		}

        private NextArg ParseOptions(ref int i, string opts)
        {
            NextArg result = NextArg.None;

            for(int j = 0; j < opts.Length; j++)
            {
                switch(opts[j])
                {
                    case 'c':
                    case 'C':
                        DumpChapters = true;
                        break;
                    case 'h':
                    case 'H':
                        UseHMS = true;
                        break;
                    case 'p':
                    case 'P':
                        result = NextArg.ProgramChain;
                        break;
                    case 'f':
                    case 'F':
                        result = NextArg.FPS;
                        break;
                    case 'l':
                    case 'L':
                        UseLongestPGC = true;
                        break;
                    case 'm':
                    case 'M':
                        result = NextArg.ChapterMinimum;
                        break;
                    case 'd':
                    case 'D':
                        AlwaysDisplayPGC = true;
                        break;
                    default:
                        UnknownOptionWarning("-" + opts[j]);
                        break;
                }
            }

            return result;
        }

        private enum NextArg
        {
            None,
            ProgramChain,
            FPS,
            ChapterMinimum
        }

        public string IFOFile;
        public bool DumpChapters;
        public bool UseHMS;
        public int PGC;
        public double FPS;
        public bool UseLongestPGC;
        public int ChapterMinimum;
        public bool AlwaysDisplayPGC;

        public bool Valid
        {
            get
            {
                return m_bValid;
            }
        }

        public void AddError(string format, params string[] args)
        {
            m_sErrors += "Error: " + string.Format(format, args);
            m_bValid = false;
        }

        public void AddWarning(string format, params string[] args)
        {
            m_sErrors += "Warning: " + string.Format(format, args);
        }

        public void UnknownOptionWarning(string sOption)
        {
            AddWarning("Unknown option: {0}", sOption);
        }

        public string Errors
        {
            get
            {
                return m_sErrors;
            }
        }
	}
}
